@@ -8,12 +8,11 @@ import (
 	"time"
 
 	"github.com/urfave/cli/v3"
-	gitalyauth "gitlab.com/gitlab-org/gitaly/v16/auth"
-	"gitlab.com/gitlab-org/gitaly/v16/internal/gitaly/config"
-	"gitlab.com/gitlab-org/gitaly/v16/internal/grpc/client"
-	"gitlab.com/gitlab-org/gitaly/v16/internal/log"
-	"gitlab.com/gitlab-org/gitaly/v16/proto/go/gitalypb"
-	"gitlab.com/gitlab-org/gitaly/v16/streamio"
+	gitalyauth "gitlab.com/gitlab-org/gitaly/v18/auth"
+	"gitlab.com/gitlab-org/gitaly/v18/internal/grpc/client"
+	"gitlab.com/gitlab-org/gitaly/v18/internal/log"
+	"gitlab.com/gitlab-org/gitaly/v18/proto/go/gitalypb"
+	"gitlab.com/gitlab-org/gitaly/v18/streamio"
 	"google.golang.org/grpc"
 )
 
@@ -72,7 +71,7 @@ func setHooksAction(ctx context.Context, cmd *cli.Command) error {
 		storage = cfg.Storages[0].Name
 	}
 
-	address, err := getAddressWithScheme(cfg)
+	address, err := cfg.GetAddressWithScheme()
 	if err != nil {
 		return fmt.Errorf("get Gitaly address: %w", err)
 	}
@@ -153,16 +152,3 @@ func dial(ctx context.Context, addr, token string, timeout time.Duration, opts .
 
 	return client.New(ctx, addr, client.WithGrpcOptions(opts))
 }
-
-func getAddressWithScheme(cfg config.Cfg) (string, error) {
-	switch {
-	case cfg.SocketPath != "":
-		return "unix:" + cfg.SocketPath, nil
-	case cfg.ListenAddr != "":
-		return "tcp://" + cfg.ListenAddr, nil
-	case cfg.TLSListenAddr != "":
-		return "tls://" + cfg.TLSListenAddr, nil
-	default:
-		return "", errors.New("no address configured")
-	}
-}
