@@ -15,27 +15,27 @@ import (
 	"github.com/prometheus/client_golang/prometheus/promauto"
 	"github.com/prometheus/client_golang/prometheus/promhttp"
 	"github.com/urfave/cli/v3"
-	"gitlab.com/gitlab-org/gitaly/v16/internal/bootstrap"
-	"gitlab.com/gitlab-org/gitaly/v16/internal/bootstrap/starter"
-	"gitlab.com/gitlab-org/gitaly/v16/internal/gitaly/config/sentry"
-	"gitlab.com/gitlab-org/gitaly/v16/internal/grpc/backchannel"
-	"gitlab.com/gitlab-org/gitaly/v16/internal/grpc/protoregistry"
-	"gitlab.com/gitlab-org/gitaly/v16/internal/grpc/sidechannel"
-	"gitlab.com/gitlab-org/gitaly/v16/internal/helper"
-	"gitlab.com/gitlab-org/gitaly/v16/internal/log"
-	"gitlab.com/gitlab-org/gitaly/v16/internal/praefect"
-	"gitlab.com/gitlab-org/gitaly/v16/internal/praefect/config"
-	"gitlab.com/gitlab-org/gitaly/v16/internal/praefect/datastore"
-	"gitlab.com/gitlab-org/gitaly/v16/internal/praefect/datastore/glsql"
-	"gitlab.com/gitlab-org/gitaly/v16/internal/praefect/metrics"
-	"gitlab.com/gitlab-org/gitaly/v16/internal/praefect/nodes"
-	"gitlab.com/gitlab-org/gitaly/v16/internal/praefect/nodes/tracker"
-	"gitlab.com/gitlab-org/gitaly/v16/internal/praefect/reconciler"
-	"gitlab.com/gitlab-org/gitaly/v16/internal/praefect/repocleaner"
-	"gitlab.com/gitlab-org/gitaly/v16/internal/praefect/service"
-	"gitlab.com/gitlab-org/gitaly/v16/internal/praefect/service/transaction"
-	"gitlab.com/gitlab-org/gitaly/v16/internal/praefect/transactions"
-	"gitlab.com/gitlab-org/gitaly/v16/internal/version"
+	"gitlab.com/gitlab-org/gitaly/v18/internal/bootstrap"
+	"gitlab.com/gitlab-org/gitaly/v18/internal/bootstrap/starter"
+	"gitlab.com/gitlab-org/gitaly/v18/internal/gitaly/config/sentry"
+	"gitlab.com/gitlab-org/gitaly/v18/internal/grpc/backchannel"
+	"gitlab.com/gitlab-org/gitaly/v18/internal/grpc/protoregistry"
+	"gitlab.com/gitlab-org/gitaly/v18/internal/grpc/sidechannel"
+	"gitlab.com/gitlab-org/gitaly/v18/internal/helper"
+	"gitlab.com/gitlab-org/gitaly/v18/internal/log"
+	"gitlab.com/gitlab-org/gitaly/v18/internal/praefect"
+	"gitlab.com/gitlab-org/gitaly/v18/internal/praefect/config"
+	"gitlab.com/gitlab-org/gitaly/v18/internal/praefect/datastore"
+	"gitlab.com/gitlab-org/gitaly/v18/internal/praefect/datastore/glsql"
+	"gitlab.com/gitlab-org/gitaly/v18/internal/praefect/metrics"
+	"gitlab.com/gitlab-org/gitaly/v18/internal/praefect/nodes"
+	"gitlab.com/gitlab-org/gitaly/v18/internal/praefect/nodes/tracker"
+	"gitlab.com/gitlab-org/gitaly/v18/internal/praefect/reconciler"
+	"gitlab.com/gitlab-org/gitaly/v18/internal/praefect/repocleaner"
+	"gitlab.com/gitlab-org/gitaly/v18/internal/praefect/service"
+	"gitlab.com/gitlab-org/gitaly/v18/internal/praefect/service/transaction"
+	"gitlab.com/gitlab-org/gitaly/v18/internal/praefect/transactions"
+	"gitlab.com/gitlab-org/gitaly/v18/internal/version"
 	"gitlab.com/gitlab-org/labkit/monitoring"
 	"gitlab.com/gitlab-org/labkit/tracing"
 )
@@ -96,6 +96,9 @@ func serveAction(ctx context.Context, cmd *cli.Command) error {
 func run(conf config.Config, appName string, logger log.Logger) error {
 	configure(logger, appName, conf)
 
+	tracerCloser := tracing.Initialize(tracing.WithServiceName(appName))
+	defer func() { _ = tracerCloser.Close() }()
+
 	starterConfigs, err := getStarterConfigs(conf)
 	if err != nil {
 		return cli.Exit(err, 1)
@@ -140,8 +143,6 @@ func readConfig(path string) (config.Config, error) {
 }
 
 func configure(logger log.Logger, appName string, conf config.Config) {
-	tracing.Initialize(tracing.WithServiceName(appName))
-
 	if conf.PrometheusListenAddr != "" {
 		conf.Prometheus.Configure(logger)
 	}
@@ -149,17 +150,12 @@ func configure(logger log.Logger, appName string, conf config.Config) {
 	sentry.ConfigureSentry(logger, version.GetVersion(), conf.Sentry)
 }
 
-func server(
-	cfgs []starter.Config,
-	conf config.Config,
-	logger log.Logger,
-	b bootstrap.Listener,
-	promreg prometheus.Registerer,
-	dbPromRegistry interface {
-		prometheus.Registerer
-		prometheus.Gatherer
-	},
-) error {
+type dbPromRegistryWrapper interface {
+	prometheus.Registerer
+	prometheus.Gatherer
+}
+
+func server(cfgs []starter.Config, conf config.Config, logger log.Logger, b bootstrap.Listener, promreg prometheus.Registerer, dbPromRegistry dbPromRegistryWrapper) error {
 	nodeLatencyHistogram, err := metrics.RegisterNodeLatency(conf.Prometheus, promreg)
 	if err != nil {
 		return err
